"cov.spatial.old" <-
function(obj, cov.model = c("exponential", "matern", "gaussian", "spherical",
                  "circular", "cubic", "wave", "powered.exponential",
                "cauchy", "gneiting", "gneiting.matern", "pure.nugget"),
         kappa, cov.pars = stop("no cov.pars argument provived"))
{
  cov.model <- match.arg(cov.model)
  if(cov.model == "matern" | cov.model == "powered.exponential" | 
     cov.model == "cauchy" | cov.model == "gneiting.matern"){
    if(missing(kappa))
      stop("for matern, powered.exponential, cauchy, gneiting and gneiting.matern covariance functions the parameter kappa must be provided"
           )
    if(cov.model == "gneiting.matern" & length(kappa) != 2)
      stop("gneiting.matern correlation function model requires a vector with 2 parameters in the argument kappa"
           )
    if((cov.model == "matern" | cov.model == "powered.exponential" | 
        cov.model == "cauchy") & length(kappa) != 1)
      stop("for this choice of  correlation function model kappa should be a scalar parameter"
           )
  }
  if(is.vector(cov.pars))
    sigmasq <- cov.pars[1]
  else sigmasq <- cov.pars[, 1]
  if(is.vector(cov.pars))
    phi <- cov.pars[2]
  else phi <- cov.pars[, 2]
  if(is.vector(cov.pars))
    ns <- 1
  else ns <- nrow(cov.pars)
  covs <- array(0, dim = dim(obj))
  for(i in 1:ns) {
    if(phi[i] < 1e-10)
      cov <- rep(0, length(obj))
    else{
      if(cov.model == "pure.nugget")
        cov <- rep(0, length(obj))
      if(cov.model == "wave")
        cov <- (1/obj) * (phi[i] * sin(obj/phi[i]))
      if(cov.model == "exponential")
        cov <- exp( - (obj/phi[i]))
      if(cov.model == "matern"){
        cov <- matern(u = obj, phi = phi[i], kappa = kappa)
      }
      if(cov.model == "gaussian")
        cov <- exp( - ((obj/phi[i])^2))
      if(cov.model == "spherical") {
        obj.sc <- obj/phi[i]
        cov <- ifelse(obj < phi[i], (1 - 1.5 * (obj.sc) + 0.5 *
                                     (obj.sc)^3), 0)
      }
      if(cov.model == "circular") {
        obj.sc <- obj/phi[i]
        obj.sc[obj.sc > 1] <- 1
        cov <- ifelse(obj < phi[i], (1 - (2 * ((obj.sc) * sqrt(
                                                               1 - ((obj.sc)^2)) + asin(obj.sc)))/pi), 0)
      }
      if(cov.model == "cubic") {
        obj.sc <- obj/phi[i]
        cov <- ifelse(obj < phi[i], (1 - (7 * (obj.sc^2) - 8.75 *
                                          (obj.sc^3) + 3.5 * (obj.sc^5) - 0.75 * (obj.sc^
                                                                                  7))), 0)
      }
      if(cov.model == "powered.exponential") {
        if(kappa > 2 | kappa <= 0)
          stop("for power model parameter phi must be in (0,2]"
               )
        cov <- exp( - ((obj/phi[i])^kappa))
      }
      if(cov.model == "cauchy") {
        cov <- (1 + (obj/phi[i])^2)^(-1 * kappa)
      }
      if(cov.model == "gneiting") {
        obj.sc <- obj/phi[i]
        t2 <- (1 - obj.sc)
        t2 <- ifelse(t2 > 0, (t2^8), 0)
        cov <- (1 + 8 * obj.sc + 25 * (obj.sc^2) + 32 * (obj.sc^
                                                         3)) * t2
      }
      if(cov.model == "gneiting.matern") {
        obj.sc <- obj/(phi[i] * kappa[2])
        t2 <- (1 - obj.sc)
        t2 <- ifelse(t2 > 0, (t2^8), 0)
        cov <- (1 + 8 * obj.sc + 25 * (obj.sc^2) + 32 * (obj.sc^
                                                         3)) * t2
        cov <- cov * matern(u = obj, phi = phi[i], kappa = 
                            kappa[1])
      }
      cov <- sigmasq[i] * cov
    }
    covs <- covs + cov
  }
  return(covs)
}
"variog.old" <-
function(geodata, coords=geodata$coords, data=geodata$data, 
         uvec = "default", trend = "cte", lambda = 1, 
         option = c("bin", "cloud", "smooth"), 
         estimator.type = c("classical", "robust"), 
         nugget.tolerance = 0, max.dist = "all", pairs.min = NULL, 
         bin.cloud = FALSE, n.points = 50, ...) 
{
  require(mva)
  require(modreg)
  call <- match.call()
  coords <- as.matrix(coords)
  data <- as.matrix(data)
  if(dim(data)[2] == 1) data <- as.vector(data)
  option <- match.arg(option)
  estimator.type <- match.arg(estimator.type)
  if (lambda != 1){
    if (lambda == 0) data <- log(data)
    else data <- ((data^lambda) - 1)/lambda
  }
  xmat <- trend.spatial(trend=trend, coords=coords)
  if(dim(xmat)[2] > 1){
    if(is.vector(data)){
      data <- lm(data ~ xmat + 0)$residuals
      names(data) <- NULL
    }
    else{
      only.res <- function(y, x){lm(y ~ x + 0)$residuals}
      data <- apply(data,2,only.res, x=xmat)
    }
  }
  u <- as.vector(dist(as.matrix(coords)))
  if (is.matrix(data)) {
    v <- matrix(0, nrow = length(u), ncol = ncol(data))
    for (i in 1:ncol(data)) {
      v[, i] <- as.vector(dist(data[, i]))
      if (estimator.type == "robust") 
        v[, i] <- v[, i]^(0.5)
      else v[, i] <- (v[, i]^2)/2
    }
    data.var <- apply(data, 2, var)
    if (is.numeric(max.dist)) {
      v <- v[u <= max.dist, ]
      u <- u[u <= max.dist]
    }
  }
  else {
    v <- as.vector(dist(data))
    if (estimator.type == "robust") 
      v <- v^(0.5)
    else v <- (v^2)/2
    data.var <- var(data)
    if (is.numeric(max.dist)) {
      v <- v[u <= max.dist]
      u <- u[u <= max.dist]
    }
  }
  if (option == "cloud") {
    result <- list(u = u, v = v)
  }
  if (option == "bin") {
    result <- rfm.bin(cloud = list(u = u, v = v),
                      estimator.type = estimator.type, 
                      uvec = uvec, nugget.tolerance = nugget.tolerance, 
                      bin.cloud = bin.cloud, ...)
    if (!is.null(pairs.min)) {
      if (is.matrix(result$v)) {
        result$v <- result$v[result$n >= pairs.min, ]
        result$sd <- result$sd[result$n >= pairs.min, 
                               ]
      }
      else {
        result$v <- result$v[result$n >= pairs.min]
        result$sd <- result$sd[result$n >= pairs.min]
      }
      result$u <- result$u[result$n >= pairs.min]
      result$n <- result$n[result$n >= pairs.min]
    }
  }
  if (option == "smooth") {
    require(modreg)
    if (is.matrix(v)) 
      stop("smooth not yet available for several variables")
    temp <- ksmooth(u, v, n.points = n.points, ...)
    result <- list(u = temp[[1]], v = temp[[2]])
  }
  result$var.mark <- data.var
  result$output.type <- option
  result$estimator.type <- estimator.type
  result$n.data <- nrow(coords)
  result$lambda <- lambda
  result$trend <- trend
  result$call <- call
  class(result) <- "variogram"
  return(result)
}
"variog.env.old" <-
function (obj.variog, coords, model.pars, trend = "cte",
          nsim = 99, messages.screen = TRUE) 
{
  call <- match.call()
  if(!is.null(model.pars$beta))
  beta <- model.pars$beta
  else beta <- 0
  if(!is.null(model.pars$cov.model))
    cov.model <- model.pars$cov.model
  else cov.model <- "exponential"
  if(!is.null(model.pars$kappa))
    kappa <- model.pars$kappa
  else kappa <- 0.5
  if(!is.null(model.pars$nugget))
    nugget <- model.pars$nugget
  else nugget <- 0
  cov.pars <- model.pars$cov.pars
  if (obj.variog$output.type != "bin") 
    stop("envelops can be computed only for binned variogram")
  if (messages.screen == TRUE) 
    print(paste("Generating", nsim, "simulations (with ", obj.variog$n.data, 
                "points each) using the function grf"))
  simula <- grf(obj.variog$n.data, cov.model=cov.model, cov.pars = cov.pars,
                nugget = nugget, kappa = kappa, nsim = nsim,
                grid = as.matrix(coords), messages.screen = FALSE)
  if(messages.screen == TRUE)
    print("Adding the trend")
  x.mat <- trend.spatial(trend=trend, coords=coords)
  mu <- as.vector(x.mat %*% beta)
  simula$data <- mu + simula$data
  if (messages.screen == TRUE) 
    print(paste("Computing the empirical variogram for the", 
                nsim, "simulations"))
  simula <- variog(coords = simula$coords, data = simula$data,
                   uvec = obj.variog$u, opt = "b", bin.cloud=F)
  if (messages.screen == TRUE) 
    print("Computing the envelops")
  limits <- apply(simula[[2]], 1, range)
  res.env <- list(u = simula$u, v.lower = limits[1, ], v.upper = limits[2, 
                                                         ])
  return(res.env)
}
